﻿using DeoVR;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;


public partial class TestForm : Form
{
    NumberFormatInfo numberFormatInfo = new NumberFormatInfo { NumberDecimalSeparator = "." };
    RemoteControlClient client = new RemoteControlClient();
    string currentPath;

    public TestForm()
    {
        InitializeComponent();
        client.OnConnected += onConnected;
        client.OnDisconnected += onDisconnected;
        client.OnDataReceived += onDataReceived;
    }

    private void onConnected()
    {
        onLogMessage("Client connected");
    }

    private void onDisconnected(Exception ex)
    {
        if (ex != null && ex.Message != null)
            onLogMessage("Error: " + ex.Message);

        onLogMessage("Client disconnected");
    }

    private void connectButton_Click(object sender, EventArgs e)
    {
        try
        {
            onLogMessage("Connecting to " + hostnameInput.Text + ":" + this.portInput.Text + "...");

            int port;
            if (!int.TryParse(this.portInput.Text, out port))
                throw new Exception("Incorrect port number!");

            client.Connect(hostnameInput.Text, port);
        }
        catch(Exception ex)
        {
            onLogMessage("Error: " + ex.Message);
        }
    }

    private void disconnectButton_Click(object sender, EventArgs e)
    {
        client.Disconnect();
    }

    void onLogMessage(string msg)
    {
        try
        {
            Invoke((MethodInvoker)delegate
            {
                msgListBox.Items.Add(msg);
                msgListBox.SelectedIndex = msgListBox.Items.Count - 1;
            });
        }
        catch
        {
        }
    }

    void onDataReceived(RemoteApiData data)
    {
        try
        {
            Invoke((MethodInvoker)delegate
            {
                if (currentPath != data.path)
                {
                    pathInput.Text = (data.path != null) ? data.path : "";
                    currentPath = data.path;
                }
                timeLabel.Text = (data.currentTime != null && data.duration != null) ? data.currentTime.Value.ToString() + " / " + data.duration.Value.ToString() : " -- ";
                stateLabel.Text = (data.playerState != null) ? data.playerState.ToString() : " -- ";
            });
        }
        catch
        {
        }
    }

    float? getCurrentTime()
    {
        float videoTime;
        if (!float.TryParse(timeInput.Text.Replace(',', '.').Trim(), NumberStyles.Float, numberFormatInfo, out videoTime))
            return null;

        return videoTime;
    }

    bool checkSeekToValue()
    {
        if (getCurrentTime() == null)
        {
            MessageBox.Show("Incorrect seek to value!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return false;
        }
        return true;
    }

    void seekToButton_Click(object sender, EventArgs e)
    {
        if (!checkSeekToValue())
            return;

        RemoteApiData timeData = new RemoteApiData { currentTime = getCurrentTime() };
        client.Send(timeData);
    }

    float? getPlaybackSpeed()
    {
        float playbackSpeed;
        if (!float.TryParse(playbackSpeedInput.Text.Replace(',', '.').Trim(), NumberStyles.Float, numberFormatInfo, out playbackSpeed))
            return null;

        return playbackSpeed;
    }

    bool checkPlaybackSpeedValue()
    {
        if (getPlaybackSpeed() == null)
        {
            MessageBox.Show("Incorrect playback speed value!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return false;
        }
        return true;
    }

    void sendPlaybackSpeedButton_Click(object sender, EventArgs e)
    {
        if (!checkPlaybackSpeedValue())
            return;

        RemoteApiData pbData = new RemoteApiData { playbackSpeed = getPlaybackSpeed() };
        client.Send(pbData);
    }

    void playButton_Click(object sender, EventArgs e)
    {
        RemoteApiData stateData = new RemoteApiData { playerState = PlayerState.Play };
        client.Send(stateData);
    }

    void pauseButton_Click(object sender, EventArgs e)
    {
        RemoteApiData stateData = new RemoteApiData { playerState = PlayerState.Pause };
        client.Send(stateData);
    }

    void openPathButton_Click(object sender, EventArgs e)
    {
        RemoteApiData stateData = new RemoteApiData { path = pathInput.Text.Trim(), currentTime = getCurrentTime(), playbackSpeed = getPlaybackSpeed() };
        client.Send(stateData);
    }

    private void sendBothButton_Click(object sender, EventArgs e)
    {
        if (!checkSeekToValue() || !checkPlaybackSpeedValue())
            return;
        RemoteApiData stateData = new RemoteApiData { currentTime = getCurrentTime(), playbackSpeed = getPlaybackSpeed() };
        client.Send(stateData);
    }

    private void TestForm_FormClosed(object sender, FormClosedEventArgs e)
    {
        client.Disconnect();
    }
}
